# -*- coding: utf-8 -*-
#
# Socratic - very simple question-answer dialogue system based on python generators.
#
# Daniil Volynkin
# foxezzz@gmail.com
#
# License: MIT
#

from socratic import Dialogue

say = print


def yes_or_no(user_message):
    if user_message.lower() == "yes":
        return True
    elif user_message.lower() == "no":
        return False
    else:
        user_message_ = yield "Please type 'yes' or 'no'"
        yield from yes_or_no(user_message_)


def ask_about_apples():
    user_message = yield "Do you like apples?"
    user_like_apples = yield from yes_or_no(user_message)
    return user_like_apples


def ask_abot_oranges():
    user_message = yield "Do you like oranges?"
    user_like_oranges = yield from yes_or_no(user_message)
    return user_like_oranges


def simple_dialogue(*_):
    user_like_apples = yield from ask_about_apples()

    if user_like_apples:
        say("I don't like it too!")
    else:
        say("Sorry to hear that. I very like it :c")

    user_like_oranges = yield from ask_abot_oranges()

    if user_like_oranges:
        say("Sorry to hear that. I hate it :c")
    else:
        say("I like it too!")

    yield from simple_dialogue()


def main():
    dialog = Dialogue(simple_dialogue)
    print("Please type something: ")

    while True:
        question = input("-- ")
        answer = dialog.say(question)
        print("--", answer)


if __name__ == '__main__':
    main()



from socratic.utils import generator_start_wrapper


def test_generator_start_wrapper():
    def test_generator():
        yield 10

    wrapped_generator = generator_start_wrapper(
        test_generator()
    )

    assert next(wrapped_generator) is None
    assert next(wrapped_generator) == 10



from socratic import Dialogue
import pytest


def test_simple_dialog():
    def entry_point(first_question):
        second_question = yield first_question

        yield second_question

    dialog = Dialogue(entry_point)

    answer_1 = dialog.say("test_message_1")
    assert answer_1 == "test_message_1"

    answer_2 = dialog.say("test_message_2")
    assert answer_2 == "test_message_2"


def test_cycled_dialog():

    def entry_point(first_question):
        if first_question == "Hello!":
            second_question = yield "Hello!"

        elif first_question == "How are you?":
            second_question = yield "Awesome!"
        else:
            second_question = yield "I don't understand you :c!"

        yield from entry_point(second_question)

    dialog = Dialogue(entry_point)

    answer_1 = dialog.say("Hello!")
    assert answer_1 == "Hello!"

    answer_2 = dialog.say("How are you?")
    assert answer_2 == "Awesome!"

    answer_3 = dialog.say("How are you?")
    assert answer_3 == "Awesome!"

    answer_4 = dialog.say("test_message")
    assert answer_4 == "I don't understand you :c!"


def test_stop_iteration():
    def entry_point(_):
        _ = yield "test1"

        _ = yield "test2"

    dialog = Dialogue(entry_point)
    answer1 = dialog.say("test")
    assert answer1 == 'test1'
    answer2 = dialog.say("test")
    assert answer2 == 'test2'
    answer3 = dialog.say("test")
    assert answer3 == 'test1'


def test_stop_iteration_on_exception():
    class TestException(Exception):
        ...

    def entry_point(_):
        _ = yield "test1"

        raise TestException()

    dialog = Dialogue(entry_point)
    answer1 = dialog.say("test")
    assert answer1 == 'test1'
    with pytest.raises(TestException):
        dialog.say("test")

    answer3 = dialog.say("test")
    assert answer3 == 'test1'

        
    






# -*- coding: utf-8 -*-
#
# Socratic - very simple question-answer dialogue system based on python generators.
#
# Daniil Volynkin
# foxezzz@gmail.com
#
# License: MIT
#

from dataclasses import dataclass
from typing import List, Optional

from socratic import Dialogue


@dataclass
class AnswerWithOptions:
    message: str
    options: Optional[List[str]] = None


def simple_dialogue(first_question):

    if first_question == "Hello!":
        second_question = yield AnswerWithOptions(
            message="Hello!",
        )
    elif first_question == "How are you?":
        second_question = yield AnswerWithOptions(
            "Awesome!",
        )
    else:
        second_question = yield AnswerWithOptions(
            "I don't understand you :c!",
            options=['Hello!', 'How are you?']
        )

    yield from simple_dialogue(second_question)


def main():
    dialog = Dialogue(simple_dialogue)
    print("Please type something: ")

    while True:
        user_replic = input('-- ')
        answer = dialog.say(user_replic)
        print("--", answer.message)
        if answer.options:
            print(answer.options)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Socratic - very simple question-answer dialogue system based on python generators.
#
# Daniil Volynkin
# foxezzz@gmail.com
#
# License: MIT
#


def generator_start_wrapper(generator):
    """
    Need for using generator.send() for first message instead next(generator).
    """
    yield

    yield from generator



# -*- coding: utf-8 -*-
#
# Socratic - very simple question-answer dialogue system based on python generators.
#
# Daniil Volynkin
# foxezzz@gmail.com
#
# License: MIT
#

from socratic import Dialogue


def simple_dialogue(first_question):
    if first_question.message == "Hello!":
        second_question = yield "Hello!"

    elif first_question.message == "How are you?":
        second_question = yield "Awesome!"
    else:
        second_question = yield (
            "I don't understand you. I answer only for next questions: "
            "['Hello!', 'How are you?']"
        )

    yield from simple_dialogue(second_question)


def main():
    dialog = Dialogue(simple_dialogue)
    print("Please type something: ")

    while True:
        question = input("-- ")
        answer = dialog.say(question)
        print("--", answer.message)


if __name__ == '__main__':
    main()





# -*- coding: utf-8 -*-
#
# Socratic - very simple question-answer dialogue system based on python generators.
#
# Daniil Volynkin
# foxezzz@gmail.com
#
# License: MIT
#
from .__version__ import (
    __title__,
    __license__,
    __version__,
    __author__,
    __contact__,
    __url__,
    __description__,
)

from .dialogue import Dialogue

__all__ = [
    Dialogue,
    __title__,
    __license__,
    __version__,
    __author__,
    __contact__,
    __url__,
    __description__,
]



# -*- coding: utf-8 -*-
#
# Socratic - very simple question-answer dialogue system based on python generators.
#
# Daniil Volynkin
# foxezzz@gmail.com
#
# License: MIT
#

from dataclasses import dataclass
from typing import Optional, Generator, Callable, Any
from inspect import getgeneratorstate, GEN_CLOSED

from socratic.utils import generator_start_wrapper


DialogueBranchState = Generator[Any, Any, None]
DialogueBranch = Callable[[Any], DialogueBranchState]


@dataclass
class Dialogue:
    entry_point: DialogueBranch
    _current_branch: Optional[DialogueBranchState] = None

    @property
    def current_branch_state(self) -> Optional[DialogueBranch]:
        if self._current_branch and self._is_current_branch_closed():
            return None

        return self._current_branch

    @current_branch_state.setter
    def current_branch_state(self, branch: DialogueBranch) -> None:
        self._current_branch = generator_start_wrapper(branch)
        next(self._current_branch)

    def reset_branch_state(self, question_message: Any) -> Optional[DialogueBranchState]:
        self.current_branch_state = self.entry_point(question_message)

        return self.current_branch_state

    def say(self, question_message: Any) -> Any:
        branch_state = self.current_branch_state

        if not branch_state:
            branch_state = self.reset_branch_state(question_message)

        try:
            answer = branch_state.send(question_message)
        except StopIteration:
            branch_state = self.reset_branch_state(question_message)
            answer = branch_state.send(question_message)

        return answer

    def _is_current_branch_closed(self) -> bool:
        return getgeneratorstate(self._current_branch) == GEN_CLOSED



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Socratic - very simple question-answer dialogue system based on python generators.
#
# Daniil Volynkin
# foxezzz@gmail.com
#
# License: MIT
#

import os
import sys
from os.path import join as pjoin
from setuptools import setup, find_packages, Command

meta = {}
with open(pjoin('socratic', '__version__.py')) as f:
    exec(f.read(), meta)

with open('README.md') as readme_file:
    readme = readme_file.read()


class Publish(Command):
    """Publish to PyPI with twine."""
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system('python setup.py sdist bdist_wheel')

        sdist = 'dist/socratic-%s.tar.gz' % meta['__version__']
        wheel = 'dist/socratic-%s-py3-none-any.whl' % meta['__version__']
        rc = os.system(
            'twine upload --verbose --repository-url=https://upload.pypi.org/legacy/ "%s" "%s"' % (sdist, wheel)
        )

        sys.exit(rc)


setup(
    name=meta['__title__'],
    license=meta['__license__'],
    version=meta['__version__'],
    author=meta['__author__'],
    author_email=meta['__contact__'],
    url=meta['__url__'],
    description=meta['__description__'],
    long_description_content_type='text/markdown',
    long_description=readme,
    platforms=['any'],
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
    ],
    install_requires=[],
    python_requires='>=3.7',
    setup_requires=[
        'pytest-runner',
    ],
    tests_require=[
        'pytest',
    ],
    cmdclass={
        'publish': Publish,
    },
)



# -*- coding: utf-8 -*-
#
# Socratic - very simple question-answer dialogue system based on python generators.
#
# Daniil Volynkin
# foxezzz@gmail.com
#
# License: MIT
#

__title__ = 'socratic'
__license__ = 'MIT'
__version__ = '0.1.1'
__author__ = 'Daniil Volynkin'
__contact__ = 'foxezzz@gmail.com'
__url__ = 'https://github.com/dvvolynkin/socratic'
__description__ = (
    'Socratic - simple question-answer dialog system.'
)
